@@ -22,22 +22,27 @@ setClass("enve.RecPlot2.Peak",
 ### of a Recruitment plot (see `enve.recplot2.findPeaks`).
    representation(
    dist='character',
-   ### Distribution of the peak. Currently supported: 'norm' (normal) and 'sn' (skew-normal).
+   ### Distribution of the peak. Currently supported: 'norm' (normal) and 'sn'
+   ### (skew-normal).
    values='numeric',
    ### Sequencing depth values predicted to conform the peak.
    values.res='numeric',
-   ### Sequencing depth values not explained by this or previously identified peaks.
+   ### Sequencing depth values not explained by this or previously identified
+   ### peaks.
    mode='numeric',
    ### Seed-value of mode anchoring the peak.
    param.hat='list',
-   ### Parameters of the distribution. A list of two values if dist='norm' (sd and mean),
-   ### or three values if dist='sn' (omega=scale, alpha=shape, and xi=location). Note that the
-   ### "dispersion" parameter is always first and the "location" parameter is always last.
+   ### Parameters of the distribution. A list of two values if dist='norm' (sd
+   ### and mean), or three values if dist='sn' (omega=scale, alpha=shape, and
+   ### xi=location). Note that the "dispersion" parameter is always first and
+   ### the "location" parameter is always last.
    n.hat='numeric',
-   ### Number of bins estimated to be explained by this peak. This should ideally be equal to
-   ### the length of `values`, but it's not and integer.
+   ### Number of bins estimated to be explained by this peak. This should
+   ### ideally be equal to the length of `values`, but it's not and integer.
    n.total='numeric',
-   ### Total number of bins from which the peak was extracted.
+   ### Total number of bins from which the peak was extracted. I.e., total
+   ### number of position bins with non-zero sequencing depth in the recruitment
+   ### plot (regardless of peak count).
    err.res='numeric',
    ### Error left after adding the peak.
    merge.logdist='numeric'
@@ -52,43 +57,50 @@ plot.enve.RecPlot2 <- function
       (x,
       ### `enve.RecPlot2` object to plot.
       layout=matrix(c(5,5,2,1,4,3), nrow=2),
-      ### Matrix indicating the position of the different panels in the layout, where:
-      ### 0: Empty space, 1: Counts matrix, 2: position histogram (sequencing depth),
-      ### 3: identity histogram, 4: Populations histogram (histogram of sequencing depths),
-      ### 5: Color scale for the counts matrix (vertical), 6: Color scale of the counts
-      ### matrix (horizontal). Only panels indicated here will be plotted. To plot only
-      ### one panel simply set this to the number of the panel you want to plot.
+      ### Matrix indicating the position of the different panels in the layout,
+      ### where:
+      ###   0: Empty space,
+      ###   1: Counts matrix,
+      ###   2: position histogram (sequencing depth),
+      ###   3: identity histogram,
+      ###   4: Populations histogram (histogram of sequencing depths),
+      ###   5: Color scale for the counts matrix (vertical),
+      ###   6: Color scale of the counts
+      ### matrix (horizontal). Only panels indicated here will be plotted. To
+      ### plot only one panel simply set this to the number of the panel you
+      ### want to plot.
       widths=c(1,7,2),
       ### Relative widths of the columns of `layout`.
       heights=c(1,2),
       ### Relative heights of the rows of `layout`.
       palette=grey((100:0)/100),
-      ### Colors to be used to represent the counts matrix, sorted from no hits to the
-      ### maximum sequencing depth.
+      ### Colors to be used to represent the counts matrix, sorted from no hits
+      ### to the maximum sequencing depth.
       underlay.group=TRUE,
-      ### If TRUE, it indicates the in-group and out-group areas couloured based on
-      ### `in.col` and `out.col`. Requires support for semi-transparency.
+      ### If TRUE, it indicates the in-group and out-group areas couloured based
+      ### on `in.col` and `out.col`. Requires support for semi-transparency.
       peaks.col='darkred',
-      ### If not NA, it attempts to represent peaks in the population histogram in the
-      ### specified color. Set to NA to avoid peak-finding.
+      ### If not NA, it attempts to represent peaks in the population histogram
+      ### in the specified color. Set to NA to avoid peak-finding.
       id.lim=range(x$id.breaks),
       ### Limits of identities to represent.
       pos.lim=range(x$pos.breaks),
       ### Limits of positions to represent (in bp, regardless of `pos.units`).
       pos.units=c('Mbp','Kbp','bp'),
-      ### Units in which the positions should be represented (powers of 1,000 base pairs).
+      ### Units in which the positions should be represented (powers of 1,000
+      ### base pairs).
       mar=list('1'=c(5,4,1,1)+.1, '2'=c(ifelse(any(layout==1),1,5),4,4,1)+.1,
-	    '3'=c(5,ifelse(any(layout==1),1,4),1,2)+0.1,
-	    '4'=c(ifelse(any(layout==1),1,5),ifelse(any(layout==2),1,4),4,2)+0.1,
-	    '5'=c(5,3,4,1)+0.1, '6'=c(5,4,4,2)+0.1),
-      ### Margins of the panels as a list, with the character representation of the number
-      ### of the panel as index (see `layout`).
+	 '3'=c(5,ifelse(any(layout==1),1,4),1,2)+0.1,
+	 '4'=c(ifelse(any(layout==1),1,5),ifelse(any(layout==2),1,4),4,2)+0.1,
+	 '5'=c(5,3,4,1)+0.1, '6'=c(5,4,4,2)+0.1),
+      ### Margins of the panels as a list, with the character representation of
+      ### the number of the panel as index (see `layout`).
       pos.splines=0,
-      ### Smoothing parameter for the splines in the position histogram. Zero (0) for no splines.
-      ### If non-zero, requires the stats package.
+      ### Smoothing parameter for the splines in the position histogram. Zero
+      ### (0) for no splines. If non-zero, requires the stats package.
       id.splines=1/2,
-      ### Smoothing parameter for the splines in the identity histogram. Zero (0) for no splines.
-      ### If non-zero, requires the stats package.
+      ### Smoothing parameter for the splines in the identity histogram. Zero
+      ### (0) for no splines. If non-zero, requires the stats package.
       in.lwd=ifelse(pos.splines>0, 1/2, 2),
       ### Line width for the sequencing depth of in-group matches.
       out.lwd=ifelse(pos.splines>0, 1/2, 2),
@@ -118,42 +130,48 @@ plot.enve.RecPlot2 <- function
    ori.mar <- par('mar');
 
    # Essential vars
-   counts	<- x$counts;
+   counts	<- x$counts
 
-   id.ingroup	<- x$id.ingroup;
-   id.counts	<- x$id.counts;
-   id.breaks	<- x$id.breaks;
-   id.mids	<- (id.breaks[-length(id.breaks)]+id.breaks[-1])/2;
-   id.binsize	<- id.breaks[-1] - id.breaks[-length(id.breaks)];
+   id.ingroup	<- x$id.ingroup
+   id.counts	<- x$id.counts
+   id.breaks	<- x$id.breaks
+   id.mids	<- (id.breaks[-length(id.breaks)]+id.breaks[-1])/2
+   id.binsize	<- id.breaks[-1] - id.breaks[-length(id.breaks)]
 
-   pos.counts.in	<- x$pos.counts.in;
-   pos.counts.out	<- x$pos.counts.out;
-   pos.breaks		<- x$pos.breaks/pos.factor;
-   pos.mids		<- (pos.breaks[-length(pos.breaks)]+pos.breaks[-1])/2;
-   pos.binsize		<- (pos.breaks[-1] - pos.breaks[-length(pos.breaks)])*pos.factor;
+   pos.counts.in  <- x$pos.counts.in
+   pos.counts.out <- x$pos.counts.out
+   pos.breaks   <- x$pos.breaks/pos.factor
+   pos.mids     <- (pos.breaks[-length(pos.breaks)]+pos.breaks[-1])/2
+   pos.binsize  <- (pos.breaks[-1] - pos.breaks[-length(pos.breaks)])*pos.factor
 
-   seqdepth.in  <- pos.counts.in/pos.binsize;
-   seqdepth.out <- pos.counts.out/pos.binsize;
-   seqdepth.lim <- range(c(seqdepth.in[seqdepth.in>0],seqdepth.out[seqdepth.out>0]))*c(1/2,2);
+   seqdepth.in  <- pos.counts.in/pos.binsize
+   seqdepth.out <- pos.counts.out/pos.binsize
+   seqdepth.lim <- range(c(seqdepth.in[seqdepth.in>0],
+		     seqdepth.out[seqdepth.out>0]))*c(1/2,2)
 
    if(underlay.group){
-      in.bg  <- do.call(rgb, c(as.list(col2rgb(in.col)), list(maxColorValue=256, alpha=62)));
-      out.bg <- do.call(rgb, c(as.list(col2rgb(out.col)[,1]), list(maxColorValue=256, alpha=52)));
+      in.bg  <- do.call(rgb, c(as.list(col2rgb(in.col)),
+		  list(maxColorValue=256, alpha=62)));
+      out.bg <- do.call(rgb, c(as.list(col2rgb(out.col)[,1]),
+		  list(maxColorValue=256, alpha=52)));
    }
 
    # Counts matrix
    if(any(layout==1)){
       par(mar=mar[['1']]);
       plot(1, t='n', bty='l',
-	 xlim=pos.lim, xlab=paste('Position in genome (',pos.units,')',sep=''), xaxs='i',
-	    ylim=id.lim,  ylab=x$id.metric, yaxs='i');
+	 xlim=pos.lim, xlab=paste('Position in genome (',pos.units,')',sep=''),
+	    xaxs='i', ylim=id.lim,  ylab=x$id.metric, yaxs='i');
       if(underlay.group){
-	 rect(pos.lim[1], id.lim[1], pos.lim[2], min(id.breaks[c(id.ingroup,TRUE)]), col=out.bg, border=NA);
-	 rect(pos.lim[1], min(id.breaks[c(id.ingroup,TRUE)]), pos.lim[2], id.lim[2], col=in.bg,  border=NA);
+	 rect(pos.lim[1], id.lim[1], pos.lim[2],
+	    min(id.breaks[c(id.ingroup,TRUE)]), col=out.bg, border=NA);
+	 rect(pos.lim[1], min(id.breaks[c(id.ingroup,TRUE)]), pos.lim[2],
+	    id.lim[2], col=in.bg,  border=NA);
       }
       abline(v=x$seq.breaks/pos.factor, col=breaks.col);
-      image(x=pos.breaks, y=id.breaks, z=log10(counts),col=palette, bg=grey(1,0),
-	 breaks=seq(-.1,log10(max(counts)), length.out=1+length(palette)), add=TRUE);
+      image(x=pos.breaks, y=id.breaks, z=log10(counts),col=palette,
+	 bg=grey(1,0), breaks=seq(-.1,log10(max(counts)),
+	 length.out=1+length(palette)), add=TRUE);
    }
 
    # Position histogram
@@ -175,11 +193,16 @@ plot.enve.RecPlot2 <- function
       lines(pos.x, rep(seqdepth.out,each=2), lwd=out.lwd, col=out.col);
       lines(pos.x, pos.f, lwd=in.lwd, col=in.col);
       if(pos.splines > 0){
-	 pos.spline <- smooth.spline(pos.x[pos.f>0], log(pos.f[pos.f>0]), spar=pos.splines)
+	 pos.spline <- smooth.spline(pos.x[pos.f>0], log(pos.f[pos.f>0]),
+	    spar=pos.splines)
 	 lines(pos.spline$x, exp(pos.spline$y), lwd=2, col=in.col)
       }
-      if(any(pos.counts.out==0)) rect(pos.breaks[c(pos.counts.out==0,FALSE)], seqdepth.lim[1], pos.breaks[c(FALSE,pos.counts.out==0)], seqdepth.lim[1]*3/2, col=out.col, border=NA);
-      if(any(pos.counts.in==0))  rect(pos.breaks[c(pos.counts.in==0,FALSE)],  seqdepth.lim[1], pos.breaks[c(FALSE,pos.counts.in==0)],  seqdepth.lim[1]*3/2, col=in.col,  border=NA);
+      if(any(pos.counts.out==0)) rect(pos.breaks[c(pos.counts.out==0,FALSE)],
+	       seqdepth.lim[1], pos.breaks[c(FALSE,pos.counts.out==0)],
+	       seqdepth.lim[1]*3/2, col=out.col, border=NA);
+      if(any(pos.counts.in==0))  rect(pos.breaks[c(pos.counts.in==0,FALSE)],
+	       seqdepth.lim[1], pos.breaks[c(FALSE,pos.counts.in==0)],
+	       seqdepth.lim[1]*3/2, col=in.col,  border=NA);
    }
 
    # Identity histogram
@@ -198,14 +221,17 @@ plot.enve.RecPlot2 <- function
 	       xlim=id.counts.range, xlab='bps per bin', xaxs='i',
 	       ylim=id.lim, yaxs='i', ylab=ylab, yaxt=yaxt);
 	 if(underlay.group){
-	    rect(id.counts.range[1], id.lim[1], id.counts.range[2], min(id.breaks[c(id.ingroup,TRUE)]), col=out.bg, border=NA);
-	    rect(id.counts.range[1], min(id.breaks[c(id.ingroup,TRUE)]), id.counts.range[2], id.lim[2], col=in.bg,  border=NA);
+	    rect(id.counts.range[1], id.lim[1], id.counts.range[2],
+	       min(id.breaks[c(id.ingroup,TRUE)]), col=out.bg, border=NA);
+	    rect(id.counts.range[1], min(id.breaks[c(id.ingroup,TRUE)]),
+	       id.counts.range[2], id.lim[2], col=in.bg,  border=NA);
 	 }
 	 id.f <- rep(id.counts,each=2)
 	 id.x <- rep(id.breaks,each=2)[-c(1,2*length(id.breaks))]
 	 lines(id.f, id.x, lwd=id.lwd, col=id.col);
 	 if(id.splines > 0){
-	    id.spline <- smooth.spline(id.x[id.f>0], log(id.f[id.f>0]), spar=id.splines)
+	    id.spline <- smooth.spline(id.x[id.f>0], log(id.f[id.f>0]),
+	       spar=id.splines)
 	    lines(exp(id.spline$y), id.spline$x, lwd=2, col=id.col)
 	 }
       }else{
@@ -225,16 +251,20 @@ plot.enve.RecPlot2 <- function
 	 ylab='Sequencing depth (X)'
 	 yaxt='s'
       }
-      h.breaks <- seq(log10(seqdepth.lim[1]*2), log10(seqdepth.lim[2]/2), length.out=200);
+      h.breaks <- seq(log10(seqdepth.lim[1]*2), log10(seqdepth.lim[2]/2),
+	 length.out=200);
       h.in <- hist(log10(seqdepth.in), breaks=h.breaks, plot=FALSE);
       h.out <- hist(log10(seqdepth.out), breaks=h.breaks, plot=FALSE);
       plot(1, t='n', log='y',
-	 xlim=range(c(h.in$counts,h.out$counts,sum(pos.counts.in==0))), xaxs='r', xlab='', xaxt='n',
-	 ylim=seqdepth.lim, yaxs='i', ylab=ylab, yaxt=yaxt);
-      y.tmp.in <- c(rep(10^h.in$breaks,each=2),seqdepth.lim[1]*c(1,1,3/2,3/2));
-      y.tmp.out <- c(rep(10^h.out$breaks,each=2),seqdepth.lim[1]*c(1,1,3/2,3/2));
-      lines(c(0,rep(h.out$counts,each=2),0,0,rep(sum(pos.counts.out==0),2),0), y.tmp.out, col=out.col);
-      polygon(c(0,rep(h.in$counts,each=2),0,0,rep(sum(pos.counts.in==0),2),0), y.tmp.in, border=NA, col=in.col);
+	 xlim=range(c(h.in$counts,h.out$counts,sum(pos.counts.in==0))),
+	 xaxs='r', xlab='', xaxt='n', ylim=seqdepth.lim, yaxs='i', ylab=ylab,
+	 yaxt=yaxt)
+      y.tmp.in <- c(rep(10^h.in$breaks,each=2),seqdepth.lim[1]*c(1,1,3/2,3/2))
+      y.tmp.out <- c(rep(10^h.out$breaks,each=2),seqdepth.lim[1]*c(1,1,3/2,3/2))
+      lines(c(0,rep(h.out$counts,each=2),0,0,rep(sum(pos.counts.out==0),2),0),
+	 y.tmp.out, col=out.col)
+      polygon(c(0,rep(h.in$counts,each=2),0,0,rep(sum(pos.counts.in==0),2),0),
+	 y.tmp.in, border=NA, col=in.col)
       if(!is.na(peaks.col)){
 	 o	<- peaks.opts; o$x = x;
 	 peaks	<- do.call(enve.recplot2.findPeaks, o);
@@ -245,13 +275,16 @@ plot.enve.RecPlot2 <- function
 	       cnt <- enve.recplot2.__peakHist(peaks[[i]], h.mids)
 	       lines(cnt, h.mids, col='red');
 	       pf <- pf+cnt;
-	       axis(4, at=peaks[[i]]$param.hat[[ length(peaks[[i]]$param.hat) ]], letters[i], las=1, hadj=1/2)
+	       axis(4, at=peaks[[i]]$param.hat[[length(peaks[[i]]$param.hat)]],
+		  letters[i], las=1, hadj=1/2)
 	    }
 	    lines(pf, h.mids, col='red',lwd=1.5);
 	    legend('bottomright', legend=paste(
 	       letters[1:length(peaks)],'. ',
-	       signif(as.numeric(lapply(peaks, function(x) tail(as.numeric(x$param.hat),n=1))),3),'X (',
-	       signif(100*as.numeric(lapply(peaks, function(x) (length(x$values)/x$n.total))), 3), '%, err: ',
+	       signif(as.numeric(lapply(peaks,
+		  function(x) tail(as.numeric(x$param.hat),n=1))),3),'X (',
+	       signif(100*as.numeric(lapply(peaks,
+		  function(x) (length(x$values)/x$n.total))), 3), '%, err: ',
 	       signif(as.numeric(lapply(peaks, function(x) x$err.res)), 3), ')',
 	       sep=''), bty='n');
 	 }
@@ -259,22 +292,28 @@ plot.enve.RecPlot2 <- function
    }
 
    # Color scale
-   count.bins <- 10^seq(log10(min(counts[counts>0])), log10(max(counts)), length.out=1+length(palette));
+   count.bins <- 10^seq(log10(min(counts[counts>0])), log10(max(counts)),
+      length.out=1+length(palette))
    if(any(layout==5)){
       par(mar=mar[['5']]);
-      plot(1,t='n',log='y',xlim=0:1,xaxt='n',xlab='',xaxs='i',ylim=range(count.bins), yaxs='i', ylab='');
-      rect(0,count.bins[-length(count.bins)],1,count.bins[-1],col=palette,border=NA);
+      plot(1,t='n',log='y',xlim=0:1,xaxt='n',xlab='',xaxs='i',
+	 ylim=range(count.bins), yaxs='i', ylab='')
+      rect(0,count.bins[-length(count.bins)],1,count.bins[-1],col=palette,
+	 border=NA)
    }
    if(any(layout==6)){
       par(mar=mar[['6']]);
-      plot(1,t='n',log='x',ylim=0:1,yaxt='n',ylab='',yaxs='i',xlim=range(count.bins), xaxs='i',xlab='');
-      rect(count.bins[-length(count.bins)],0,count.bins[-1],1,col=palette,border=NA);
+      plot(1,t='n',log='x',ylim=0:1,yaxt='n',ylab='',yaxs='i',
+	 xlim=range(count.bins), xaxs='i',xlab='');
+      rect(count.bins[-length(count.bins)],0,count.bins[-1],1,col=palette,
+	 border=NA);
    }
    
    par(mar=ori.mar);
    return(peaks);
-   ### Returns a list of `enve.RecPlot2.Peak` objects (see `enve.recplot2.findPeaks`). If `peaks.col`=NA or
-   ### `layout` doesn't include 4, returns NA.
+   ### Returns a list of `enve.RecPlot2.Peak` objects (see
+   ### `enve.recplot2.findPeaks`). If `peaks.col`=NA or `layout` doesn't include
+   ### 4, returns NA.
 }
 
 #==============> Define core functions
@@ -325,9 +364,10 @@ enve.recplot2 <- function(
    # Build matrix
    if(verbose) cat("Building counts matrix.\n")
    if(id.metric=="corrected identity" & ncol(rec)<6){
-      stop("Requesting corrected identity, but .rec file doesn't have 6th column")
+      stop("Requesting corr. identity, but .rec file doesn't have 6th column")
    }
-   rec.idcol <- ifelse(id.metric=="identity", 3, ifelse(id.metric=="corrected identity", 6, 4));
+   rec.idcol <- ifelse(id.metric=="identity", 3,
+      ifelse(id.metric=="corrected identity", 6, 4));
    if(length(pos.breaks)==1){
       if(pos.breaks>0){
          pos.breaks <- seq(min(lim[,2]), max(lim[,3]), length.out=pos.breaks+1);
@@ -341,6 +381,7 @@ enve.recplot2 <- function(
    }
    
    # Run in parallel
+   if(nrow(rec) < 200) threads <- 1 # It doesn't worth the overhead
    cl		<- makeCluster(threads)
    rec.l	<- list()
    thl		<- ceiling(nrow(rec)/threads)
@@ -384,37 +425,48 @@ enve.recplot2.findPeaks <- function(
    ### sub-population mixtures.
       x,
       ### An `enve.RecPlot2` object.
-      min.points=50,
-      ### Minimum number of points in the quantile-estimation-range (`quant.est`) to estimate a peak.
-      quant.est=c(0.005, 0.995),
-      ### Range of quantiles to be used in the estimation of a peak's parameters.
+      min.points=10,
+      ### Minimum number of points in the quantile-estimation-range
+      ### (`quant.est`) to estimate a peak.
+      quant.est=c(0.002, 0.998),
+      ### Range of quantiles to be used in the estimation of a peak's
+      ### parameters.
       mlv.opts=list(method='parzen'),
       ### Options passed to `mlv` to estimate the mode.
-      fitdist.opts.sn=list(distr='sn', method='qme', probs=c(.1, .5, .8), start=list(omega=1, alpha=-1), lower=c(1e-6, -Inf, 0), upper=c(Inf, 0, Inf)),
-      ### Options passed to `fitdist` to estimate the standard deviation if with.skewness=TRUE. Note that
-      ### the `start` parameter will be ammended with xi=estimated mode for each peak.
-      fitdist.opts.norm=list(distr='norm', method='qme', probs=c(.4,.6), start=list(sd=1), lower=c(1e-8, 0)),
-      ### Options passed to `fitdist` to estimate the standard deviation if with.skewness=FALSE. Note that
-      ### the `start` parameter will be ammended with mean=estimated mode for each peak.
+      fitdist.opts.sn=list(distr='sn', method='qme', probs=c(0.1,0.5,0.8),
+	 start=list(omega=1, alpha=-1), lower=c(1e-6, -Inf, 0),
+	 upper=c(Inf, 0, Inf)),
+      ### Options passed to `fitdist` to estimate the standard deviation if
+      ### with.skewness=TRUE. Note that the `start` parameter will be ammended
+      ### with xi=estimated mode for each peak.
+      fitdist.opts.norm=list(distr='norm', method='qme', probs=c(.4,.6),
+	 start=list(sd=1), lower=c(1e-8, 0)),
+      ### Options passed to `fitdist` to estimate the standard deviation if
+      ### with.skewness=FALSE. Note that the `start` parameter will be ammended
+      ### with mean=estimated mode for each peak.
       rm.top=0.05,
-      ### Top-values to remove before finding peaks, as a quantile probability. This step
-      ### is useful to remove highly conserved regions, but can be turned off by setting
-      ### rm.top=0. The quantile is determined *after* removing zero-coverage windows.
+      ### Top-values to remove before finding peaks, as a quantile probability.
+      ### This step is useful to remove highly conserved regions, but can be
+      ### turned off by setting rm.top=0. The quantile is determined *after*
+      ### removing zero-coverage windows.
       with.skewness=TRUE,
-      ### Allow skewness correction of the peaks. Typically, the sequencing-depth distribution
-      ### for a single peak is left-skewed, due partly (but not exclusively) to fragmentation
-      ### and mapping sensitivity. See Lindner et al 2013, Bioinformatics 29(10):1260-7 for
-      ### an alternative solution for the first problem (fragmentation) called "tail
-      ### distribution".
+      ### Allow skewness correction of the peaks. Typically, the
+      ### sequencing-depth distribution for a single peak is left-skewed, due
+      ### partly (but not exclusively) to fragmentation and mapping sensitivity.
+      ### See Lindner et al 2013, Bioinformatics 29(10):1260-7 for an
+      ### alternative solution for the first problem (fragmentation) called
+      ### "tail distribution".
       optim.rounds=200,
       ### Maximum rounds of peak optimization.
       optim.epsilon=1e-8,
-      ### Trace change at which optimization stops (unless `optim.rounds` is reached first). The
-      ### trace change is estimated as the sum of square differences between parameters in one round and
-      ### those from two rounds earlier (to avoid infinite loops from approximation).
-      merge.logdist=log(1.25),
-      ### Maximum value of |log-ratio| between centrality parameters in peaks to attempt merging. The default
-      ### of ~0.22 corresponds to a maximum difference of 25%.
+      ### Trace change at which optimization stops (unless `optim.rounds` is
+      ### reached first). The trace change is estimated as the sum of square
+      ### differences between parameters in one round and those from two rounds
+      ### earlier (to avoid infinite loops from approximation).
+      merge.logdist=log(1.75),
+      ### Maximum value of |log-ratio| between centrality parameters in peaks to
+      ### attempt merging. The default of ~0.22 corresponds to a maximum
+      ### difference of 25%.
       verbose=FALSE
       ### Display (mostly debugging) information.
    ){
@@ -424,10 +476,15 @@ enve.recplot2.findPeaks <- function(
    seqdepth.in	<- x$pos.counts.in/pos.binsize;
    lsd1 <- seqdepth.in[seqdepth.in>0];
    lsd1 <- lsd1[ lsd1 < quantile(lsd1, 1-rm.top, names=FALSE) ]
-   if(with.skewness){ fitdist.opts <- fitdist.opts.sn }else{ fitdist.opts <- fitdist.opts.norm }
-   peaks.opts <- list(lsd1=lsd1, min.points=min.points, quant.est=quant.est, mlv.opts=mlv.opts,
-      fitdist.opts=fitdist.opts, with.skewness=with.skewness, optim.rounds=optim.rounds,
-      optim.epsilon=optim.epsilon, verbose=verbose, n.total=length(lsd1), merge.logdist=merge.logdist)
+   if(with.skewness){
+      fitdist.opts <- fitdist.opts.sn
+   }else{
+      fitdist.opts <- fitdist.opts.norm
+   }
+   peaks.opts <- list(lsd1=lsd1, min.points=min.points, quant.est=quant.est,
+      mlv.opts=mlv.opts, fitdist.opts=fitdist.opts, with.skewness=with.skewness,
+      optim.rounds=optim.rounds, optim.epsilon=optim.epsilon, verbose=verbose,
+      n.total=length(lsd1), merge.logdist=merge.logdist)
    
    # Find seed peaks
    if(verbose) cat('Mowing peaks for n =',length(lsd1),'\n')
@@ -445,8 +502,12 @@ enve.recplot2.findPeaks <- function(
 	 p <- peaks[[ i ]]
 	 j <- enve.recplot2.__whichClosestPeak(p, peaks)
 	 p2 <- peaks[[ j ]]
-	 if( abs(log(p$param.hat[[ length(p$param.hat) ]]/p2$param.hat[[ length(p2$param.hat) ]])) < merge.logdist ){
-	    if(verbose) cat('==> Attempting a merge at',p$param.hat[[ length(p$param.hat) ]],'&',p2$param.hat[[ length(p2$param.hat) ]],'X\n');
+	 dst.a <- p$param.hat[[ length(p$param.hat) ]]
+	 dst.b <- p2$param.hat[[ length(p2$param.hat) ]]
+	 if( abs(log(dst.a/dst.b)) < merge.logdist ){
+	    if(verbose) cat('==> Attempting a merge at',
+		  p$param.hat[[ length(p$param.hat) ]],'&',
+		  p2$param.hat[[ length(p2$param.hat) ]],'X\n');
 	    peaks.opts$lsd1 <- c(p$values, p2$values)
 	    p.new <- enve.recplot2.__findPeaks(peaks.opts)
 	    if(length(p.new)==1){
@@ -466,6 +527,7 @@ enve.recplot2.findPeaks <- function(
    ### Returns a list of `enve.RecPlot2.Peak` objects.
 }
 
+#==============> Define utils
 enve.recplot2.corePeak <- function
    ### Finds the peak in a list of peaks that is most likely to represent the
    ### "core genome" of a population.
@@ -484,14 +546,35 @@ enve.recplot2.corePeak <- function
       sz.d = log(length(p$values)/length(corePeak$values))
       if(sz.d < 0)
 	 next;
-      sq.d = log(p$param.hat[[ length(p$param.hat) ]]/maxPeak$param.hat[[ length(maxPeak$param.hat) ]] )
-      if(abs(sq.d) < maxPeak$merge.logdist+sz.d/5)
+      sq.d.a <- p$param.hat[[ length(p$param.hat) ]]
+      sq.d.b <- maxPeak$param.hat[[ length(maxPeak$param.hat) ]]
+      if(abs(log(sq.d.a/sq.d.b )) < maxPeak$merge.logdist+sz.d/5)
          corePeak <- p
    }
    return(corePeak)
 }
 
+enve.recplot2.changeCutoff <- function
+   ### Change the intra-species cutoff of an existing recruitment plot.
+      (rp,
+      ### enve.RecPlot2 object.
+      new.cutoff=98
+      ### New cutoff to use.
+      ){
+   # Re-calculate vectors
+   id.mids	<- (rp$id.breaks[-length(rp$id.breaks)]+rp$id.breaks[-1])/2
+   id.ingroup	<- (id.mids > new.cutoff)
+   pos.counts.in  <- apply(rp$counts[,id.ingroup], 1, sum)
+   pos.counts.out <- apply(rp$counts[,!id.ingroup], 1, sum)
+   # Update object
+   attr(rp, "id.ingroup")     <- id.ingroup
+   attr(rp, "pos.counts.in")  <- pos.counts.in
+   attr(rp, "pos.counts.out") <- pos.counts.out
+   attr(rp, "call")           <- match.call()
+   return(rp)
+}
 
+#==============> Define internal functions
 enve.recplot2.__counts <- function
    ### Internal ancilliary function (see `enve.recplot2`).
       (x, pos.breaks, id.breaks, rec.idcol){
@@ -500,9 +583,12 @@ enve.recplot2.__counts <- function
    counts <- matrix(0, nrow=length(pos.breaks)-1, ncol=length(id.breaks)-1);
    for(i in 1:nrow(rec)){
       if(verbose & i%%100==0) cat("   [",signif(i*100/nrow(rec),3),"% ]   \r");
-      y.bin <- which(rec[i,rec.idcol]>=id.breaks[-length(id.breaks)] & rec[i,rec.idcol]<=id.breaks[-1])[1] ;
+      y.bin <- which(
+	 rec[i,rec.idcol]>=id.breaks[-length(id.breaks)] &
+	 rec[i,rec.idcol]<=id.breaks[-1])[1] ;
       for(pos in rec[i,1]:rec[i,2]){
-	 x.bin <- which(pos>=pos.breaks[-length(pos.breaks)] & pos<=pos.breaks[-1])[1] ;
+	 x.bin <- which(
+	    pos>=pos.breaks[-length(pos.breaks)] & pos<=pos.breaks[-1])[1] ;
 	 counts[x.bin, y.bin] <- counts[x.bin, y.bin]+1 ;
       }
    }
@@ -537,11 +623,12 @@ enve.recplot2.__findPeak <- function
    
    # Refine peak parameters
    for(round in 1:optim.rounds){
-      param.hat[[ 1 ]] <- param.hat[[ 1 ]]/diff(quant.est) # <- expand dispersion
+      param.hat[[ 1 ]] <- param.hat[[ 1 ]]/diff(quant.est)# <- expand dispersion
       lim.o <- param.hat
       lim.o$p <- quant.est; lim <- do.call(paste('q',dist,sep=''), lim.o)
       lsd1.pop <- lsd1[(lsd1>lim[1]) & (lsd1<lim[2])];
-      if(verbose) cat(' Round', round, 'with n =',length(lsd1.pop),'and params =',as.numeric(param.hat),' \r')
+      if(verbose) cat(' Round', round, 'with n =',length(lsd1.pop),
+	    'and params =',as.numeric(param.hat),' \r')
       if(length(lsd1.pop) < min.points) break;
       o <- fitdist.opts; o$data = lsd1.pop; o$start = param.hat;
       last.last.hat <- last.hat
@@ -551,7 +638,8 @@ enve.recplot2.__findPeak <- function
 	 if(round>1) param.hat <- last.hat;
 	 break;
       }
-      if(round>2) if(sum((as.numeric(last.last.hat)-as.numeric(param.hat))^2) < optim.epsilon) break;
+      epsilon <- sum((as.numeric(last.last.hat)-as.numeric(param.hat))^2)
+      if(round>2) if(epsilon < optim.epsilon) break;
    }
    if(verbose) cat('\n')
    if(is.na(param.hat[1]) | is.na(lim[1])) return(NULL);
@@ -560,10 +648,12 @@ enve.recplot2.__findPeak <- function
    lsd2 <- c();
    lsd.pop <- c();
    n.hat <- length(lsd1.pop)/diff(quant.est)
-   peak <- new('enve.RecPlot2.Peak', dist=dist, values=as.numeric(), mode=mode1, param.hat=param.hat,
-      n.hat=n.hat, n.total=n.total, merge.logdist=merge.logdist)
+   peak <- new('enve.RecPlot2.Peak', dist=dist, values=as.numeric(), mode=mode1,
+      param.hat=param.hat, n.hat=n.hat, n.total=n.total,
+      merge.logdist=merge.logdist)
    peak.breaks <- seq(min(lsd1), max(lsd1), length=20)
-   peak.cnt <- enve.recplot2.__peakHist(peak, (peak.breaks[-length(peak.breaks)]+peak.breaks[-1])/2)
+   peak.cnt <- enve.recplot2.__peakHist(peak,
+      (peak.breaks[-length(peak.breaks)]+peak.breaks[-1])/2)
    for(i in 2:length(peak.breaks)){
       values <- lsd1[ (lsd1 >= peak.breaks[i-1]) & (lsd1 < peak.breaks[i]) ]
       n.exp <- peak.cnt[i-1]
@@ -578,8 +668,11 @@ enve.recplot2.__findPeak <- function
    # Return peak
    attr(peak, 'values') <- lsd.pop
    attr(peak, 'values.res') <- lsd2
-   attr(peak, 'err.res') <- 1-(cor(hist(lsd.pop, breaks=peak.breaks, plot=FALSE)$counts, hist(lsd1, breaks=peak.breaks, plot=FALSE)$counts)+1)/2
-   if(verbose) cat(' Extracted peak with n =',length(lsd.pop),'with expected n =',n.hat,'\n')
+   attr(peak, 'err.res') <- 1-(cor(hist(lsd.pop, breaks=peak.breaks,
+      plot=FALSE)$counts, hist(lsd1, breaks=peak.breaks,
+      plot=FALSE)$counts)+1)/2
+   if(verbose) cat(' Extracted peak with n =',length(lsd.pop),
+	 'with expected n =',n.hat,'\n')
    return(peak)
 }
 
